/* Tabela RESERPRODPED: Reserva de Produtos para o Pedido */
/* Descrio: A trigger insere registros na tabela LOGOPERA a partir de alteraes na tabela RESERPRODPED */
/* Objetivo: Gerar um log que vai registrar as alteraes na tabela Reserva de Produtos para o Pedido,
             utilizando a tabela Geral de Logs de Operaes */

IF EXISTS(SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'LOG_PED_RESERPRODPED' and
            type = 'TR'                     and
            u.uid = o.uid                   and
            u.name = USER)
  DROP TRIGGER LOG_PED_RESERPRODPED
  GO

CREATE TRIGGER LOG_PED_RESERPRODPED
ON RESERPRODPED
FOR INSERT, UPDATE, DELETE
AS
   DECLARE @V_USUARIO        VARCHAR(40);
   DECLARE @V_TIPO           VARCHAR(1);
   DECLARE @V_ITPESQ         VARCHAR(40);
   DECLARE @V_DESC           VARCHAR(300);
   DECLARE @V_CDFILLOGOPER   VARCHAR(4);
   DECLARE @V_DTOPER         DATETIME;
   DECLARE @V_CDARVPROD      VARCHAR(20);
   DECLARE @V_NMPRODUTO      VARCHAR(50);

/* VARIVEIS DO CURSOR C_CURSOR*/
   DECLARE @V_ID_COMANDO       VARCHAR(1);
 
   DECLARE @V_NRLOTEESTQ_OLD   VARCHAR(20);
   DECLARE @V_CDPRODUTO_OLD    VARCHAR(10);
   DECLARE @V_CDFILIAL_OLD     VARCHAR(4);
   DECLARE @V_QTRESPRODPED_OLD double precision;
   DECLARE @V_NRPEDIDO_OLD     VARCHAR(10);
   DECLARE @V_CDALMOXARIFE_OLD VARCHAR(6);
   DECLARE @V_CDLOCALESTOQ_OLD VARCHAR(6);
   DECLARE @V_NRSUBLOTE_OLD    VARCHAR(4);

   DECLARE @V_NRLOTEESTQ_NEW   VARCHAR(20);
   DECLARE @V_CDPRODUTO_NEW    VARCHAR(10);
   DECLARE @V_CDFILIAL_NEW     VARCHAR(4);
   DECLARE @V_QTRESPRODPED_NEW double precision;
   DECLARE @V_NRPEDIDO_NEW     VARCHAR(10);
   DECLARE @V_CDALMOXARIFE_NEW VARCHAR(6);
   DECLARE @V_CDLOCALESTOQ_NEW VARCHAR(6);
   DECLARE @V_NRSUBLOTE_NEW    VARCHAR(4);
BEGIN

   DECLARE C_CURSOR CURSOR FOR
      SELECT
         'D' IDCOMANDO,

         NULL,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL,  

         NRLOTEESTQ,
         CDPRODUTO,
         CDFILIAL,
         QTRESPRODPED,
         NRPEDIDO,
         CDALMOXARIFE,
         CDLOCALESTOQ,
         NRSUBLOTE
      FROM
         DELETED
      WHERE
         0 = (SELECT COUNT(*) FROM INSERTED)

      UNION ALL

      SELECT
         'U' IDCOMANDO,

         INSERTED.NRLOTEESTQ,
         INSERTED.CDPRODUTO,
         INSERTED.CDFILIAL,
         INSERTED.QTRESPRODPED,
         INSERTED.NRPEDIDO,
         INSERTED.CDALMOXARIFE,
         INSERTED.CDLOCALESTOQ,
         INSERTED.NRSUBLOTE,

         DELETED.NRLOTEESTQ,
         DELETED.CDPRODUTO,
         DELETED.CDFILIAL,
         DELETED.QTRESPRODPED,
         DELETED.NRPEDIDO,
         DELETED.CDALMOXARIFE,
         DELETED.CDLOCALESTOQ,
         DELETED.NRSUBLOTE
      FROM
         INSERTED,
         DELETED
      WHERE
         INSERTED.CDFILIAL  = DELETED.CDFILIAL

      UNION ALL
 
      SELECT
         'I' IDCOMANDO,

         INSERTED.NRLOTEESTQ,
         INSERTED.CDPRODUTO,
         INSERTED.CDFILIAL,
         INSERTED.QTRESPRODPED,
         INSERTED.NRPEDIDO,
         INSERTED.CDALMOXARIFE,
         INSERTED.CDLOCALESTOQ,
         INSERTED.NRSUBLOTE,

         NULL,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL
      FROM
         INSERTED
      WHERE
         0 = (SELECT COUNT(*) FROM DELETED)

   OPEN C_CURSOR;
   FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO, 
 
                                 @V_NRLOTEESTQ_OLD,
                                 @V_CDPRODUTO_OLD,
                                 @V_CDFILIAL_OLD,
                                 @V_QTRESPRODPED_OLD,
                                 @V_NRPEDIDO_OLD,
                                 @V_CDALMOXARIFE_OLD,
                                 @V_CDLOCALESTOQ_OLD,
                                 @V_NRSUBLOTE_OLD,

                                 @V_NRLOTEESTQ_NEW,
                                 @V_CDPRODUTO_NEW,
                                 @V_CDFILIAL_NEW,
                                 @V_QTRESPRODPED_NEW,
                                 @V_NRPEDIDO_NEW,
                                 @V_CDALMOXARIFE_NEW,
                                 @V_CDLOCALESTOQ_NEW,
                                 @V_NRSUBLOTE_NEW;

   WHILE (@@FETCH_STATUS = 0) BEGIN
      IF @V_ID_COMANDO = 'I' BEGIN
         SELECT @V_CDARVPROD = CDARVPROD,
                @V_NMPRODUTO = NMPRODUTO
         FROM PRODUTO
         WHERE CDPRODUTO = @V_CDPRODUTO_NEW;

         SET @V_DESC   = 'Filial '    + @V_CDFILIAL_NEW     + ' ' +
                         'Pedido '    + @V_NRPEDIDO_NEW     + ' ' +
                         'Produto '   + @V_CDARVPROD        + ' ' +
                         'Qtde. '     + CONVERT(CHAR, ROUND(@V_QTRESPRODPED_NEW, 3));

         IF (@V_CDALMOXARIFE_NEW IS NOT NULL) OR
            (@V_CDLOCALESTOQ_NEW IS NOT NULL) OR
            (@V_NRLOTEESTQ_NEW   IS NOT NULL) OR
            (@V_NRSUBLOTE_NEW    IS NOT NULL) BEGIN
            SET @V_DESC =  @V_DESC + ' ' + 'Estoque ';
            IF (@V_CDALMOXARIFE_NEW IS NOT NULL) BEGIN
               SET @V_DESC =  @V_DESC + ' ' + @V_CDALMOXARIFE_NEW;
            END;
            IF (@V_CDLOCALESTOQ_NEW IS NOT NULL) BEGIN
               SET @V_DESC =  @V_DESC + ' ' + @V_CDLOCALESTOQ_NEW;
            END;
            IF (@V_NRLOTEESTQ_OLD IS NOT NULL) BEGIN
               SET @V_DESC =  @V_DESC + ' ' + @V_NRLOTEESTQ_NEW;
            END;
            IF (@V_NRSUBLOTE_NEW IS NOT NULL) BEGIN
               SET @V_DESC =  @V_DESC + ' ' + @V_NRSUBLOTE_NEW;
            END;
         END;

         SET @V_ITPESQ  = @V_CDFILIAL_NEW+@V_NRPEDIDO_NEW+@V_CDARVPROD;
         SET @V_TIPO = '1';
         SET @V_CDFILLOGOPER = @V_CDFILIAL_NEW;
      END
      ELSE BEGIN
         SELECT @V_CDARVPROD = CDARVPROD,
                @V_NMPRODUTO = NMPRODUTO
         FROM PRODUTO
         WHERE CDPRODUTO = @V_CDPRODUTO_OLD;

         SET @V_DESC   = 'Filial '    + @V_CDFILIAL_OLD     + ' ' +
                         'Pedido '    + @V_NRPEDIDO_OLD     + ' ' +
                         'Produto '   + @V_CDARVPROD        + ' ' +
                         'Qtde. '     + CONVERT(CHAR, ROUND(@V_QTRESPRODPED_OLD, 3));

         IF (@V_CDALMOXARIFE_OLD IS NOT NULL) OR
            (@V_CDLOCALESTOQ_OLD IS NOT NULL) OR
            (@V_NRLOTEESTQ_OLD   IS NOT NULL) OR
            (@V_NRSUBLOTE_OLD    IS NOT NULL) BEGIN
            SET @V_DESC =  @V_DESC + ' ' + 'Estoque ';
            IF (@V_CDALMOXARIFE_OLD IS NOT NULL) BEGIN
               SET @V_DESC =  @V_DESC + ' ' + @V_CDALMOXARIFE_OLD;
            END;
            IF (@V_CDLOCALESTOQ_OLD IS NOT NULL) BEGIN
               SET @V_DESC =  @V_DESC + ' ' + @V_CDLOCALESTOQ_OLD;
            END;
            IF (@V_NRLOTEESTQ_OLD IS NOT NULL) BEGIN
               SET @V_DESC =  @V_DESC + ' ' + @V_NRLOTEESTQ_OLD;
            END;
            IF (@V_NRSUBLOTE_OLD IS NOT NULL) BEGIN
               SET @V_DESC =  @V_DESC + ' ' + @V_NRSUBLOTE_OLD;
            END;
         END;

         SET @V_ITPESQ  = @V_CDFILIAL_OLD+@V_NRPEDIDO_OLD+@V_CDARVPROD;
         SET @V_CDFILLOGOPER = @V_CDFILIAL_OLD;
         IF @V_ID_COMANDO = 'U' BEGIN
            SET @V_TIPO = '2';
         END
         ELSE BEGIN
            IF @V_ID_COMANDO = 'D'  BEGIN
               SET @V_TIPO = '3';
            END;
         END;
      END;

      EXEC NMUSUARIO @V_USUARIO;
      SET @V_USUARIO = ISNULL(@V_USUARIO, '');
      IF NOT @V_DESC IS NULL BEGIN
         IF NOT @V_USUARIO IS NULL BEGIN
            SET @V_DTOPER = GETDATE();

             INSERT INTO LOGOPERA
                  (NMTABELA      , DTOPERACAO, NMRESPOPER,
                   IDTIPOOPER, CDPESQITEM, DSOPERACAO, CDFILLOGOPER)
             VALUES
                  ('RESERPRODPED', GETDATE(), @V_USUARIO,
                   @V_TIPO   , @V_ITPESQ ,@V_DESC    , @V_CDFILLOGOPER);
         END;
      END;
      FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,

                                    @V_NRLOTEESTQ_OLD,
                                    @V_CDPRODUTO_OLD,
                                    @V_CDFILIAL_OLD,
                                    @V_QTRESPRODPED_OLD,
                                    @V_NRPEDIDO_OLD,
                                    @V_CDALMOXARIFE_OLD,
                                    @V_CDLOCALESTOQ_OLD,
                                    @V_NRSUBLOTE_OLD,

                                    @V_NRLOTEESTQ_NEW,
                                    @V_CDPRODUTO_NEW,
                                    @V_CDFILIAL_NEW,
                                    @V_QTRESPRODPED_NEW,
                                    @V_NRPEDIDO_NEW,
                                    @V_CDALMOXARIFE_NEW,
                                    @V_CDLOCALESTOQ_NEW,
                                    @V_NRSUBLOTE_NEW;

   END;
   CLOSE      C_CURSOR;
   DEALLOCATE C_CURSOR;
END;
GO
